function r = P_2_nearest_autocorrelation(rhat, Gamma)

% Compute (nonnegative) autocorrelation vector that is nearest to a given vector
% Input:
%   rhat   - experimental correlation vector
%   Gamma  - weights matrix
%  Output:
%   r      - autocorrelation vector nearest to rhat
%
% Objective: minimize (r-rhat)^T Gamma (r-rhat), s.t. r positive.
 
% BD 10.10.2000, 2.05.2010

rhat = rhat(:);
n = length(rhat);
n1 = n + 1;
 
if size(Gamma,1) ~= n || size(Gamma,2) ~= n
  error('Wrong size of Gamma')
end
 
U = chol(Gamma);
rc = U * rhat;
U = inv(U)';

K.q = [n1];
K.s = [n];

% free term
c = zeros(n1+n*n,1);
c(2:n1,1) = rc;

% objective coefficient
b = zeros(n1,1);
b(1) = -1;

% LMI coefficients
A = zeros( n1 + n*n , n1 );
A(1,1) = -1;
A(2:n1,2:n1) = - U;

i1 = n1+1;
i2 = n1+n*n;
e = ones(n,1);
A(i1:i2,2) = vec( -eye(n) );
for i = 2:n
  A(i1:i2,i+1) = vec( -diag(e(1:n-i+1), i-1) );
end

pars.fid = 0;
pars.eps = 1e-8;
%t = cputime;
[x,y,info] = sedumi(A,b,c,K,pars);
info
%t = cputime - t
 
X = mat(x(i1:i2));

% only for test
r = rhat + Gamma \ y(2:n+1);
