function [h,u] = cfr(r, nreg)
% Optimum compaction filter with regularity constraints
%
% [h,u] = cfr(r, nreg)
%
% Input:
%   r	      - correlation vector
%   nreg    - number of regularity constraints
% Output:
%   h	- optimum compaction filter
%   u	- factorization of h (regularity zeros removed)

% BD 13.10.2000
% Modified 21.06.2006

r = r / r(1);
n = length(r);		% Toeplitz matrix size
N = n - 1;
m = floor((n-1)/2) + 1;	% number of variables ("free" diagonals),
                  			% plus 1, the max eigenvalue variable
nv = N - nreg;		% size of V (causal part)
nv1 = nv + 1;
na = nv1 * nv1;

K.s = [nv1];

% the free term of the LMI
comb = pascal(nreg+1, 1);
comb = abs(comb(nreg+1, :));
NR = zeros(n, n+nreg);
for i=1:n
  NR(i,i:i+nreg) = comb;
end
NR = NR(:,nreg+1:n);
C = - NR' * toeplitz(r) * NR;
C = toeplitz( C(:,1) );	% enforce symmetry !
c = sparse(na,1);
c = vec( C );

% the objective
b = sparse(m,1);
b(1) = -1;		% for the "eigenvalue"

% the coefficients of the LMI
A = sparse(na, m);
for i = 0:m-1
  j = 2*i + 1;
  rr = zeros(1,n);
  rr(j) = 1;
  a = - toeplitz(rr);
  a = NR' * a * NR;
  a = toeplitz( a(:,1) );
  A(:,i+1) = sparse(vec(a));
end

pars.fid = 0;
pars.eps = 1e-12;
[x,y,info] = sedumi(A,b,c,K,pars);
info
%constr_error_g = norm(b - A'*x)
 
X = mat(x);
g = zeros(nv1,1);
for k = 1 : nv1
  g(k) = sum(diag(X,k-1));
end
%g = g / g(1);
u = factsp_orch(g);


% add regularity zeros
comb = pascal(nreg+1,1);
comb = abs(comb(nreg+1,:));

h = conv(u,comb);

h = h/norm(h);
