% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Examples 3.17, 3.18, 3.19: minimum value of multivariate trigonometric
% polynomial via sum-of-squares relaxation.

% Variables:
%   R - polynomial coefficients, symmetric matrix (all coefficients)
%   nplus - extra degree (the degree of the relaxation is nh+nplus)

ex_number = 1;    % set this to 1, 2 or 3
use_grampair = 0; % set to 1 to call the function based on the Gram pair parameterization
                  % (section 3.9)
use_cvx = 0;      % set to 1 to call CVX-based function
nplus = [0 0];    % set to e.g. [1 1] for a potentially better approximation,
                  % but increasing the complexity

switch ex_number
  case 1,     % example 3.17
    R = [1 1 0; 0 5 0; 0 1 1];
            
  case 2,     % example 3.18
    R = [-5     4     1
         -8    18     2
          1    38     1
          2    18    -8
          1     4    -5];

  case 3,     % example 3.19
    R = [-0.125    0.5   0.25    0.5  -0.125
          0.5      1     1       1     0.5
          0.25     1     3.5     1     0.25
          0.5      1     1       1     0.5
         -0.125    0.5   0.25    0.5  -0.125];
end

% pad with zeros if so desired
if sum(nplus)
  Rsize = size(R) + 2*nplus;
  Rbig = zeros(Rsize);
  Rbig(1+nplus(1):end-nplus(1),1+nplus(2):end-nplus(2)) = R;
  R = Rbig;
end

% compute minimum value
if use_cvx
  nR = (size(R)-1) / 2;  % degree of the polynomial
  Rvec = vec(R);
  M = length(Rvec);
  Rvec = Rvec((M+1)/2:end);  % vector of coefficients (halfplane of causal part)
  mu = minpol_cvx(Rvec, nR);
elseif use_grampair  % Gram pair parameterization
  mu = minpol2_g2(R);
else   % basic parameterization
  [Q, smallest_eig, mu] =  most_pos_Gram(R);
end
mu
