% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Examples 4.19, 4.20: minimum value of multivariate trigonometric polynomial
% on a frequency domain, via sum-of-squares relaxation.
% The domain can be a union of sets, each defined by the positivity of some
% polynomials; however, here the domain is defined by a single polynomial
% (see ex_5_4_6 for a full use of this kind of constrained optimization)

% Variables:
%   R - polynomial coefficients, symmetric matrix (all coefficients)
%   D - the polynomials defining the domain
%   nplus - extra degree (the degree of the relaxation is nh+nplus)

pol_number = 2;   % set to 1, 2 or 3 to choose the polynomial to minimize
dom_number = 1;   % set to 1 or 2 to choose the domain
use_cvx = 0;      % set to 1 to call CVX-based function
nplus = [0 0];    % set to e.g. [1 1] for a potentially better approximation,
                  % but increasing the complexity

switch pol_number
  case 1,     % not discussed in the book, see the unconstrained case in example 3.17
    R = [1 1 0; 0 5 0; 0 1 1];
            
  case 2,     % example 4.18
    R = [-5     4     1
         -8    18     2
          1    38     1
          2    18    -8
          1     4    -5];

  case 3,     % example 4.19
    R = [-0.125    0.5   0.25    0.5  -0.125
          0.5      1     1       1     0.5
          0.25     1     3.5     1     0.25
          0.5      1     1       1     0.5
         -0.125    0.5   0.25    0.5  -0.125];
end

switch dom_number
  case 1,     % domain (4.21)
    nD = 1;   % number of polynomials (here: one domain in the union, one polynomial for the domain)
    D{1} =  [0     0.5   0;
             0.5  -1     0.5;
             0     0.5   0];  
  case 2,     % complementary of (4.21)
    nD = 1;
    D{1} = -[0     0.5   0;
             0.5  -1     0.5;
             0     0.5   0];  
end

% compute minimum value
if use_cvx 
  if sum(nplus) % pad with zeros if necessary
    Rsize = size(R) + 2*nplus;
    Rbig = zeros(Rsize);
    Rbig(1+nplus(1):end-nplus(1),1+nplus(2):end-nplus(2)) = R;
    R = Rbig;
  end
  nR = (size(R)-1) / 2;  % degree of the polynomial
  Rvec = vec(R);
  M = length(Rvec);
  Rvec = Rvec((M+1)/2:end);  % vector of coefficients (halfplane of causal part)
  for i = 1 : length(D)   % convert polynomials to form required by minpol_cvx
    Dstruct{i}.d = (size(D{i})-1)/2;
    Dvec = vec(D{i});
    M = length(Dvec);
    Dstruct{i}.p = Dvec((M+1)/2:end);
  end
  mu = minpol_cvx(Rvec, nR, 1, nD, Dstruct);
else   % basic parameterization
  mu = minpol2_dom(R, nD, D, nplus);
end
mu
