% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 5.2: design of lowpass FIR filter with arbitrary phase.
% The stopband energy is minimized subject to bounds on the passband
% and stopband magnitude error.

% Design data:
n = 50;            % filter order (must be even)
wp = 0.2;          % passband edge (normalized frequency)
ws = 0.25;         % stopband edge
gp = 0.1;          % passband error bound
gs = 10^(-40/20);  % stopband error bound (-40dB)

use_cvx = 0;       % set to 1 for the CVX-based implementation

% Optimize filter
if use_cvx
  cvx_solver sedumi  % SDPT3 solves it inaccurately !
  [h,Es] = fir_lowpass_minEs_mag_cvx(n, wp, ws, gp, gs);
  cvx_solver sdpt3
else
  [h,Es] = fir_lowpass_minEs_mag(n, wp, ws, gp, gs);
end

% Draw frequency response
freqz_lowpass_mask(h, 1, wp, ws, gp, gs, 1);
axis([0 1 -80 10])
%fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
%set(hh,'FontSize',fontsize);
hh = ylabel('Magnitude (dB)');
%set(hh,'FontSize',fontsize);
