% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 5.3: design of lowpass FIR filter with approximately linear phase.
% The stopband energy is minimized subject to bounds on the passband
% and stopband magnitude error

% Design data:
n = 50;            % filter order (must be even)
tau = 22;          % desired group delay
wp = 0.2;          % passband edge (normalized frequency)
ws = 0.25;         % stopband edge
gp = 0.1;          % passband error bound
gs = 10^(-36/20);  % stopband error bound (-36dB)

use_cvx = 0;       % set to 1 for the CVX-based implementation

% Optimize filter
if use_cvx
  cvx_solver sedumi  % SDPT3 cannot solve it !
  [h,Es] = fir_lowpass_minEs_mag_alin_cvx(n, tau, wp, ws, gp, gs);
  cvx_solver sdpt3;
else
  [h,Es] = fir_lowpass_minEs_mag_alin(n, tau, wp, ws, gp, gs);
end

% Draw frequency response
freqz_lowpass_mask(h, 1, wp, ws, gp, gs, 1);
axis([0 1 -80 10])
%fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
%set(hh,'FontSize',fontsize);
hh = ylabel('Magnitude (dB)');
%set(hh,'FontSize',fontsize);

% Draw group delay in passband
w = (0:0.0001:wp)*pi;
gd = grpdelay(h,1,w);
figure
plot(w/pi, gd);
grid
hh = xlabel('Frequency (\omega / \pi)');
%set(hh,'FontSize',fontsize);
hh = ylabel('Group delay');
%set(hh,'FontSize',fontsize);

