% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Examples 5.4, 5.5, 5.6: design of linear phase 2D FIR filters.
% The stopband absolute error is minimized subject to
% a bound on the passband error.

% Variables (see fir2_inf_lin for details):
%   nh - filter degree
%   nplus - extra degree (the degree of the relaxation is nh+nplus)
%   gp - passband error bound
%   nDp, nDs - number of polynomials describing the passband, stopband
%   Dp, Ds - polynomials describing the passband, stopband

ex_number = 1; % set this to 1, 2 or 3
use_cvx = 0;       % set to 1 for the CVX-based implementation

nh = [7 7];
nplus = [0 0];

switch ex_number
  case 1,     % example 5.4, simplest lowpass
    gp = 0.05;
    nDp = 1;
    nDs = 1;
    c1 = 1.0;
    c2 = 0.3;
    Dp{1} = [0    0.5   0;
             0.5 -c1    0.5;
             0    0.5   0];
    Ds{1} = [ 0    -0.5   0;
             -0.5   c2   -0.5;
              0    -0.5   0];
            
  case 2,     % example 5.5, diamond lowpass
    gp = 0.1;
    nDp = 3;
    nDs = [1 1 1];
    c1 = 0;
    c2 = -0.7;
    Dp{1} = [0.5   0   0;
             0    -c1  0;
             0     0   0.5];
    Dp{2} = [0     0   0.5;
             0    -c1  0;
             0.5   0   0];
    Dp{3} = [0    0.5   0;
             0.5  0     0.5;
             0    0.5   0];
    Ds{1} =-[0.5   0   0;
             0    -c2  0;
             0     0   0.5];
    Ds{2} =-[0     0   0.5;
             0    -c2  0;
             0.5   0   0];
    Ds{3} =-[0    0.5   0;
             0.5  0     0.5;
             0    0.5   0];

  case 3,     % example 5.6, fan
    gp = 0.1;
    nDp = 2;
    nDs = [1 1];
    alfa1 = [1.0 0];
    alfa2 = [0. -0.7];
    Dp{1} = [ 0      1       0
             -0.5 -alfa1(1) -0.5
              0      1       0];
    Dp{2} = [ 0.5 -alfa1(2)  0.5 ];
    Ds{1} =-[ 0      1       0
             -0.5 -alfa2(1) -0.5
              0      1       0];
    Ds{2} =-[ 0.5 -alfa2(2)  0.5 ];
end

% call design function
if use_cvx
  cvx_solver sedumi   % SDPT3 is inaccurate or fails
  [h,gs] = fir2_inf_lin_cvx(nh, gp, Dp, nDp, Ds, nDs, nplus);
  cvx_solver sdpt3
else
  [h,gs] = fir2_inf_lin(nh, gp, Dp, nDp, Ds, nDs, nplus);
end

% draw frequency response (size of h is 2*nh+1)
freqz_2d(h,1);
