% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 5.7: design of approximately linear phase 2D FIR filters.

ex_number = 1;     % set this to 1, 2 or 3
use_cvx = 1;       % only CVX version for this example

nh = [10 10];   % filter degree
nplus = [0 0];  % extra degree (the degree of the relaxation is nh+nplus)
tau = [4 4];    % group delay
gp = 0.1;

switch ex_number
  case 1,       % ex. 5.7, same constraints as in ex. 5.4
    nDp = 1;
    nDs = 1;
    c1 = 1.0;
    c2 = 0.3;
    Dp{1} = [0    0.5   0;
             0.5 -c1    0.5;
             0    0.5   0];
    Ds{1} = [ 0    -0.5   0;
             -0.5   c2   -0.5;
              0    -0.5   0];
  case 2,     % like in example 5.5, diamond lowpass (not reported in the book)
    nDp = 3;
    nDs = [1 1 1];
    c1 = 0;
    c2 = -0.7;
    Dp{1} = [0.5   0   0;
             0    -c1  0;
             0     0   0.5];
    Dp{2} = [0     0   0.5;
             0    -c1  0;
             0.5   0   0];
    Dp{3} = [0    0.5   0;
             0.5  0     0.5;
             0    0.5   0];
    Ds{1} =-[0.5   0   0;
             0    -c2  0;
             0     0   0.5];
    Ds{2} =-[0     0   0.5;
             0    -c2  0;
             0.5   0   0];
    Ds{3} =-[0    0.5   0;
             0.5  0     0.5;
             0    0.5   0];

  case 3,     % like in example 5.6, fan (not reported in the book)
    nDp = 2;
    nDs = [1 1];
    alfa1 = [1.0 0];
    alfa2 = [0. -0.7];
    Dp{1} = [ 0      1       0
             -0.5 -alfa1(1) -0.5
              0      1       0];
    Dp{2} = [ 0.5 -alfa1(2)  0.5 ];
    Ds{1} =-[ 0      1       0
             -0.5 -alfa2(1) -0.5
              0      1       0];
    Ds{2} =-[ 0.5 -alfa2(2)  0.5 ];
end

% call design function
cvx_solver sedumi   % SDPT3 is inaccurate or fails
[h, gs] = fir2_inf_alin_cvx(nh, tau, gp, Dp, nDp, Ds, nDs, nplus);
cvx_solver sdpt3

% draw frequency response (size of h is 2*nh+1)
freqz_2d(h,1);
