% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 5.8: FIR deconvolution (standard setup)

% channel model (cell vector)
G{1} = 1;       % in general, the coefficients can be matrices
G{2} = 0.33562;
G{3} = 4.627;
G{4} = -0.14487;
G{5} = 1.6837;

% desired model - delay z^{-2}
D{1} = 0;
D{2} = 0;
D{3} = 1;

% optimization criterion
crit = 2;  % 0  - infinity norm
           % 1  - quadratic norm
           % 2  - mixed (minimize quadratic norm subject to infinity norm bound

% filter design data
Nx = 6;        % degree
gamma = 0.45;  % Hinf norm bound (used only if crit==2)
    

% design 
[X, err, E] = fir_deconv(G, D, Nx, crit, gamma, [], []);

% make vectors instead of cells, if scalar system
if size(G{1},1) == 1    
  x = zeros(Nx+1,1);
  e = zeros(length(E),1);
  for k = 1 : Nx+1
    x(k) = X{k}(1);
  end
  for k = 1 : length(E)
    e(k) = E{k}(1);
  end
end

% recompute norms (check with Table 5.2)
err2 = norm_fir_2(E)
errinf = norm_fir_mimo_brl(E)
