% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 5.9: FIR deconvolution for periodic filters

% channel model (period is 2)
H0 = [5 1 2 -1];
H1 = [3 2 -2 1];
G{1} = [ H0(1) H0(2); 0 H1(1) ];
G{2} = [ H0(3) H0(4); H1(2) H1(3) ];
G{3} = [ 0 0; H1(4) 0];

% desired model - delay z^{-2}
for i=1:5
  D{i}=zeros(2);
end
D{1}=[1 0; 0 1];

% optimization criterion
crit = 2;  % 0  - infinity norm
           % 1  - quadratic norm
           % 2  - mixed (minimize quadratic norm subject to infinity norm bound

% filter design data
deg=3;              % degree of the periodic filter
Nx = ceil(deg/2);   % degree of MIMO filter
gamma = 0.74;       % Hinf norm bound (used only if crit==2)
    
% build linear constraints corresponding to MIMO filter
Nx1 = Nx+1;
if rem(deg,2) == 1
  lincA = zeros(4,4*Nx1);
  lincA(1,2)=1; lincA(2,4*Nx1-3)=1; lincA(3,4*Nx1-1)=1; lincA(4,4*Nx1)=1;
  lincb=zeros(4,1);
else
  lincA = zeros(2,4*Nx1);
  lincA(1,2)=1; lincA(2,4*Nx1-1)=1;
  lincb=zeros(2,1);
end

% design
[X, err, E] = fir_deconv(G, D, Nx, crit, gamma, lincA, lincb);

% optimal filter
X0 = [X{1}(1,1) X{1}(1,2) X{2}(1,1) X{2}(1,2)]
X1 = [X{1}(2,2) X{2}(2,1) X{2}(2,2) X{3}(2,1)]

% recompute norms
err2 = norm_fir_2(E)/sqrt(2)
errinf = norm_fir_mimo_brl(E)
