% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 6.11: GDFT modulated oversampled filter bank

% Design data
n = 47;        % prototype degree
M = 8;         % number of channels (covering [0, 2*pi])
L = 6;         % downsampling factor
gd = 0.000001; % distortion error bound
gs = 1;        % stopband error bound (ignored if >= 1, like in the book)
               % try e.g. gs = 10^(-24/20) to see some effects
               
% optimization of the prototype
[h,Es,g] = gdft_orth(M, n, L, gd, gs);

% draw prototype response
[H,w] = freqz(h);
H = 20*log10(abs(H));
w = w / pi;
plot(w, H)
axis([0 1 -70 10])
grid
title('Prototype response')
xlabel('Frequency (\omega / \pi)');
ylabel('Magnitude (dB)');

% draw FB response (from -pi to pi)
figure
nn = (0:n)';
for m = 0 : M-1
  hm = h .* exp( j*pi*(2*m+1)*nn/M);
  [Hm,w] = freqz(hm,1,-pi:pi/500:pi);
  Hm = 20*log10(abs(Hm));
  plot(w/pi, Hm)
  hold on
end
hold off
axis([-1 1 -70 10])
grid
title('FB frequency responses')
xlabel('Frequency (\omega / \pi)');
ylabel('Magnitude (dB)');

% draw distortion error
figure
Td = zeros(2*n + 1, 1);
Td( n+1+M : M : end ) = M/L * g(1+M : M : end);
Td( n+1-M : -M : 1 ) = M/L * g(1+M : M : end);
[T,w] = freqz(Td);
plot(w/pi, abs(T))
grid
title('Distortion error')
xlabel('Frequency (\omega / \pi)');
ylabel('Magnitude');

