% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Examples 6.1, 6.2: two-channel filter bank adapted to a signal

% Design data
n = 19;     % filter order (must be odd)
nreg = 2;   % number of roots at -1 (0 for ex. 6.1, 4 for ex.6.2)

% Autocorrelation sequence (replace with any positive sequence)
rho = 0.9;  % AR(1) with the pole rho
r = ones(n+1,1);
r(2:end) = rho;
r = cumprod(r);

% Design program
h = cfr(r, nreg);

% Draw frequency response
[H,w] = freqz(h);
H = 20*log10(abs(H));
w = w / pi;
plot(w, H)              % first filter
hold on
plot(w(end:-1:1), H)   % second filter
axis([0 1 -80 10])
grid
xlabel('Frequency (\omega / \pi)');
ylabel('Magnitude (dB)');

% The corresponding scaling function
if nreg > 0
  figure
  [f,x] = dilation(h,8);
  plot(x,f)
  grid
  xlabel('Time');
  ylabel('\phi(t)');
end
