% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Examples 7.1, 7.3: multidimensional stability test

ex_number = 1;   % set to 1 or 2 for systems with denominators (7.7) or (7.8)
sparse_base = 2; % type of base used for the Gram matrix
                 % 0 - complete base (might be slow)
                 % 1 - minimal (use only monomials with nonzero coefficients)
                 % 2 - "divisors" (use only divisors of the monomials with nonzero coefficients)

switch ex_number
  case 1,   % system (7.7)
    nvar = 5;           % number of variables
    ndeg = [3 1 3 2 1]; % maximum degrees of variables

    polcoef = zeros(ndeg+1);  % nvar-dimensional array of coefficients
    polcoef(1,1,1,1,1) = 5;
    polcoef(3,1,4,1,1) = 1;
    polcoef(1,1,4,3,1) = 1;
    polcoef(4,2,1,1,2) = 1;
    polcoef(2,2,2,2,2) = 1;
    
  case 2,   % system (7.8)
    nvar = 3;       % number of variables
    ndeg = [2 3 2]; % maximum degrees of variables
    polcoef = zeros(ndeg+1);
    polcoef(1,1,1) = 5.6;
    polcoef(2,1,1) = 0.8;
    polcoef(3,2,1) = 1.5;
    polcoef(1,4,1) = 1.8;
    polcoef(1,1,2) = 0.2;
    polcoef(1,2,3) = 1.3;
end

% call stability test
[is_stable, mineig, gap] = stab_nd_syst(polcoef, sparse_base);

% show results
if is_stable
  fprintf('System is stable:\n')
else
  fprintf('System is not stable:\n')
end
fprintf('  Smallest eigenvalue of most positive Gram matrix = %g\n', mineig)
fprintf('  Gap = %g\n', gap)
