% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 7.4: stability test for Fornasini-Marchesini model

nplus = 0;         % extra degree for relaxation

% model matrices
A1 = [ 0.5 -0.1 0.2 ; 0.1 -0.2 0.3 ; -0.2 0.1 0.3];
A2 = [ 0.1 -0.4 0.2 ; 0.1 -0.4 0.3 ; -0.2 0.2 0.2];
A3 = [ 0.2 -0.3 0.2 ; 0.2 -0.2 0.3 ; -0.1 0.4 0.3];

% call stability test
[is_stable, mineig, gap] = stab_fm(A1, A2, A3, nplus);

% show results
if is_stable
  fprintf('System is stable:\n')
else
  fprintf('System is not stable:\n')
end
fprintf('  Smallest eigenvalue of most positive Gram matrix = %g\n', mineig)
fprintf('  Gap = %g\n', gap)

% test based on characteristic polynomial (with scalar coefficients
polcoef = fm_charpoly(A1,A2,A3);
[is_stable1, mineig1, gap1] = stab_nd_syst(polcoef);
if is_stable1
  fprintf('Characteristic polynomial (scalar) is stable:\n')
else
  fprintf('Characteristic polynomial (scalar) is not stable:\n')
end
fprintf('  Smallest eigenvalue of most positive Gram matrix = %g\n', mineig1)
fprintf('  Gap = %g\n', gap1)
