% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 8.4: design of approximately linear phase IIR filters with
% fixed denominator

% Design data
m = 50;    % numerator order
tau = 22;  % desired grooup delay
wp = 0.2;  % passband edge
ws = 0.25; % stopband edge
gp = 0.1;  % passband error bound
r = 0.8*exp(sqrt(-1)*0.2*pi);  % denominator roots
a = poly([r conj(r)]);

% Discretized ideal response and weights
L = 200;
df = 1/L;  % discretization step
freqp = (0:df:wp) * pi;
freqs = (ws:df:1) * pi;
freq = [freqp freqs];
Hid = [exp(-sqrt(-1)*tau*freqp) zeros(1, length(freqs))];  % ideal response
weightls = [ ones(1, length(freqp)) 100*ones(1, length(freqs))];

% LS design
b1 = iir_ls_fixa( m, a, freq, Hid, weightls);
ls_criterion = J2(weightls, Hid, freq, b1, a)

% Minimax design
[b2, gs] = iir_lowpass_minmax_alin_fixa(m, a, tau, wp, ws, gp);
stopband_error = gs

% Draw frequency responses
[H1, fg] = freqz(b1, a);
H2 = freqz(b2, a, fg);
plot(fg/pi, 20*log10(abs(H1)),'--');
hold on
plot(fg/pi, 20*log10(abs(H2)));
hold off
grid
axis([0 1 -80 10])
xlabel('Frequency (\omega / \pi)');
ylabel('Magnitude (dB)');
