% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 8.7: design of approximately linear phase IIR filters

% Design data
m = 50;    % numerator order
n = 2;     % denominator order
tau = 22;  % desired grooup delay
rho = 0.8; % max pole radius
wp = 0.2;  % passband edge
ws = 0.25; % stopband edge
gp = 0.1;  % passband error bound

% Discretized ideal response and weights
L = 200;
df = 1/L;  % discretization step
freqp = (0:df:wp) * pi;
freqs = (ws:df:1) * pi;
freq = [freqp freqs];
Hid = [exp(-sqrt(-1)*tau*freqp) zeros(1, length(freqs))];  % ideal response
weightls = [ ones(1, length(freqp)) 100*ones(1, length(freqs))];

% LS design, SMGNR
[a1, b1, crit1] = iir_sm_prstab(n, m, rho, freq, Hid, weightls);  % Steiglitz-McBride
[a2, b2, crit2] = iir_gn_prstab(n, m, rho, freq, Hid, weightls, a1, b1);  % Gauss-Newton
[a3, b3, crit3] = iir_newton_prstab(n, m, rho, freq, Hid, weightls, a2, b2);  % descent

% LS design, GN only (gives better results in this case)
[a4, b4, crit4] = iir_gn_prstab(n, m, rho, freq, Hid, weightls);  % Gauss-Newton directly
