% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2007

% Example 8.9: design of approximately linear phase 2D IIR filters
% with circular shape

% design data
n1 = 4;    % denominator order
n2 = 4;
m1 = 12;   % numerator order
m2 = 12;
tau = 7;   % desired group delay (the same on both directions)
rho = 0.9; % maximum pole radius
wp = 0.5;  % passband radius (normalized)
ws = 0.7;  % stopband radius (normalized)

% frequency grid for discretized criterion
L1 = 40;
L2 = 40;
freq1 = -pi : pi/L1 : pi;
freq2 = 0 : pi/L2 : pi;

% desired response
Hid = zeros(length(freq1), length(freq2));
k1 = freq1 / pi;
k2 = freq2 / pi;
linphase = repmat(freq1',1,length(freq2))+repmat(freq2,length(freq1),1);
absf = sqrt( repmat(k1'.*k1',1,length(freq2)) + ...
                 repmat(k2.*k2, length(freq1), 1) );

bandfreq = find( absf <= wp );
stopfreq = find( absf >= ws );
Hid(bandfreq) = exp(-sqrt(-1)*tau*linphase(bandfreq));

%weights
weight = zeros(length(freq1),length(freq2));
weight(bandfreq) = 1;
weight(stopfreq) = 1;

if freq2(1) == 0      % weight correction for half grid
  weight(:,1) = 0.5 * weight(:,1);      % half weights on axis
  weight(L1+1,1) = 2 * weight(L1+1,1);  % but not in the origin
end

% initialize with optimal FIR filter
aa = zeros(n1+1,n2+1);
aa(1,1) = 1;
bb = iir2_ls_fixa( m1, m2, aa, freq1, freq2, Hid, weight);

% Design with Gauss-Newton
[agn,bgn,crit0] = iir2_gn_prstab(aa, bb, rho, 0, freq1, freq2, Hid, weight);

if  freq2(1) == 0         % adjust criterion
  l1 = length(freq1);
  l2 = length(freq2);
  crit0 = crit0 * (l1*l2) / ((l1*(2*l2-1)+1)/2);
end

% draw frequency response
freqz_2d(bgn,agn)
