% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 9.3: compute the atomic norm of a sum of complex sinusoids

% BD 2.12.2016

n = 31;   % signal length is n+1
w = [0.2 0.3 0.6]*pi;   % frequencies
c = [1.5 -2 2.5];       % coefficients

% generate the sum
x = sum( repmat(c',1,n+1) .* exp(1i*w'*(0:n)) ).';

% solve SDP problem for computing the atomic norm using Pos3Poly
p = [n 1];      % degree and coefficients size (scalars)
ptype = [];
ptype.complex_coef = 1;
ptype.brl.hsize = 1;
cvx_begin
  variable HR(2*(n+1)) complex;
  maximize( real(x'*HR(1:n+1) ) )
  subject to
    HR == sos_pol(p, ptype);
    HR(n+2:end) == eye(n+1,1);
cvx_end

% draw graph of trigonometric polynomial to see the frequencies
h = HR(1:n+1);      % coefficients of the polynomial
f = 0:pi/1000:2*pi;
hf = zeros(length(f),1);
for i = 1 : length(f)
  a = exp(1i*f(i)*(0:n)');
  hf(i) = abs(a'*h);
end
plot(f/pi, hf, 'k', 'LineWidth', 2)
grid
hold on
for i = 1 : length(w)
  plot( [w(i) w(i)]/pi, [0 1], 'r' )
end
hold off

fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
set(hh,'FontSize',fontsize);
hh = ylabel('$|H(\omega)|$','Interpreter','latex');
set(hh,'FontSize',fontsize);

%print -deps 'fig_atom_norm.eps'

% check primal SDP for computing the (same) atomic norm
cvx_begin
  variable lam(n+1) complex;
  variable zeta;
  variable Z(n+1,n+1) complex;
  minimize( 0.5*(real(lam(1)) + zeta) )
  subject to
    [Z x; x' zeta] == hermitian_semidefinite(n+2);
    for i=0:n       % Z is Toeplitz, quite inefficient
      for k=0:n-i
        Z(1+k,i+1+k) == lam(i+1);
      end
    end
cvx_end


