% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 9.5: compute the matrix atomic norm of two sums of complex sinusoids
% with the same frequencies

% BD 6.12.2016

n = 15;   % signal length is n+1
m = 2;    % number of signals. Works only for m=2 !!!!
w = [0.2 0.3 0.6]*pi;   % frequencies
c = [1.5 -2 2.5];      % coefficients
c1 = c;      % coefficients
c2 = [2 1 -1.5];        % only amplitude difference
c2 = c1*exp(1i*pi/3);   % only phase difference
c2 = [2*exp(1i*0.5*pi) 1*exp(1i*0.25*pi) -1.5*exp(1i*1.8*pi)];   % both

% generate the sums
x1 = sum( repmat(c1',1,n+1) .* exp(1i*w'*(0:n)) ).';
x2 = sum( repmat(c2',1,n+1) .* exp(1i*w'*(0:n)) ).';
X = [x1 x2];

% solve SDP problem for computing the atomic norm using Pos3Poly
p = [n 1];      % degree and coefficients size (scalars)
ptype = [];
ptype.complex_coef = 1;
ptype.brl.hsize = [1 2];
cvx_begin
  variable HR((m+1)*(n+1)) complex;
  maximize( real(trace(X'*[HR(1:m:m*(n+1)) HR(2:m:m*(n+1))]) ) )
  subject to
    HR == sos_pol(p, ptype);
    HR(m*(n+1)+1:end) == eye(n+1,1);
cvx_end

% draw graph of trigonometric polynomial to see the frequencies
H = [HR(1:m:m*(n+1)) HR(2:m:m*(n+1))];      % coefficients of the matrix polynomial
f = 0:pi/1000:2*pi;
HF = zeros(length(f),m);
for i = 1 : length(f)
  a = exp(1i*f(i)*(0:n)');
  HF(i,:) = abs(a'*H);
end
plot(f/pi,sqrt(HF(:,1).*HF(:,1)+HF(:,2).*HF(:,2)), 'k', 'LineWidth', 2);
grid
hold on
plot(f/pi,HF(:,1), '--')
plot(f/pi,HF(:,2), 'g:')
for i = 1 : length(w)
  plot( [w(i) w(i)]/pi, [0 1], 'r' )
end
hold off

fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
set(hh,'FontSize',fontsize);
hh = ylabel('$\|\boldmath{H}(\omega)\|$','Interpreter','latex');
set(hh,'FontSize',fontsize);

%print -deps 'fig_atom_norm_mat.eps'

