% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 9.6: line spectrum estimatiom with atomic norm

% BD 2.12.2016

n = 31;   % signal length is n+1
w = [0.2 0.3 0.6]*pi;   % frequencies
c = [1.5 -2 2.5];       % coefficients
gamma = 4;
sigma = 0.1;            % noise variance

% generate the sum, add noise
xtrue = sum( repmat(c',1,n+1) .* exp(1i*w'*(0:n)) ).';
xhat = xtrue + sigma * (randn(n+1,1) + 1i*randn(n+1,1));

% solve SDP problem for line spectrum estimation using Pos3Poly
p = [n 1];      % degree and coefficients size (scalars)
ptype = [];
ptype.complex_coef = 1;
ptype.brl.hsize = 1;
cvx_begin
  variable HR(2*(n+1)) complex;
  minimize( 0.5*(HR(1:n+1)-xhat)'*(HR(1:n+1)-xhat) - real(xhat'*HR(1:n+1) ) )
  subject to
    HR == sos_pol(p, ptype);
    HR(n+2:end) == gamma*gamma*eye(n+1,1);
cvx_end

% draw graph of trigonometric polynomial to see the frequencies
h = HR(1:n+1);      % coefficients of the polynomial
f = 0:pi/1000:2*pi;
hf = zeros(length(f),1);
for i = 1 : length(f)
  a = exp(1i*f(i)*(0:n)');
  hf(i) = abs(a'*h);
end
plot(f/pi,hf)
grid
hold on
for i = 1 : length(w)
  plot( [w(i) w(i)]/pi, [0 gamma], 'r' )
end
hold off

fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
set(hh,'FontSize',fontsize);
hh = ylabel('$|H(\omega)|$','Interpreter','latex');
set(hh,'FontSize',fontsize);

%print -deps 'fig_line_spectrum.eps'

