% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 9.6, matrix version (not discussed in the book, but similar to ex.9.5):
% line spectrum estimatiom with atomic norm

% BD 2.12.2016

n = 31;   % segment length is n+1
m = 2;    % number of segments; works only for m=2!!!
w = [0.2 0.3 0.6]*pi;   % frequencies
c = [1.5 -2 2.5];       % coefficients
gamma = 10;
sigma = 0.1;            % noise variance

% generate the sum, add noise
xtrue = sum( repmat(c',1,m*(n+1)) .* exp(1i*w'*(0:m*(n+1)-1)) ).';
xhat = xtrue + sigma * (randn(m*(n+1),1) + 1i*randn(m*(n+1),1));

Xhat = reshape(xhat,n+1,m);

% solve SDP problem for line spectrum estimation using Pos3Poly
p = [n 1];      % degree and coefficients size (scalars)
ptype = [];
ptype.complex_coef = 1;
ptype.brl.hsize = [1 2];
cvx_begin
  variable HR((m+1)*(n+1)) complex;
  minimize( 0.5*real(([HR(1:m:m*(n+1)); HR(2:m:m*(n+1))]-xhat)' * ([HR(1:m:m*(n+1)); HR(2:m:m*(n+1))]-xhat)) ...
           - real(xhat'*[HR(1:m:m*(n+1)); HR(2:m:m*(n+1))]) )
  subject to
    HR == sos_pol(p, ptype);
    HR(m*(n+1)+1:end) == gamma*gamma*eye(n+1,1);
cvx_end

% draw graph of trigonometric polynomial to see the frequencies
H = [HR(1:m:m*(n+1)) HR(2:m:m*(n+1))];      % coefficients of the matrix polynomial
f = 0:pi/1000:2*pi;
HF = zeros(length(f),m);
for i = 1 : length(f)
  a = exp(1i*f(i)*(0:n)');
  HF(i,:) = abs(a'*H);
end
plot(f/pi,sqrt(HF(:,1).*HF(:,1)+HF(:,2).*HF(:,2)));
grid
hold on
for i = 1 : length(w)
  plot( [w(i) w(i)]/pi, [0 gamma], 'r' )
end
hold off

fontsize = 20;
hh = xlabel('Frequency (\omega / \pi)');
set(hh,'FontSize',fontsize);
hh = ylabel('$\|\boldmath{H}(\omega)\|$','Interpreter','latex');
set(hh,'FontSize',fontsize);

%print -deps 'fig_line_spectrum_mat.eps'

