% Programs for the book
% "Positive Trigonometric Polynomials and Signal Processing Applications"
% by Bogdan Dumitrescu, Springer 2017 (second edition)

% Example 9.8: compute the atomic norm of a sum of 2D complex sinusoids

% BD 14.12.2016

n = 7;   % signal length is n+1
w1 = [0.2 0.3 0.6]*pi;   % frequencies
w2 = [0.7 -0.5 0.1]*pi;
c = [1.5 -2 2.5];       % coefficients

% generate the sum
n2 = (n+1)*(n+1);
x = zeros(n2,1);
for i = 1 : length(c)
  x = x + c(i) * kron(exp(1i*w2(i)*(0:n)'), exp(1i*w1(i)*(0:n)'));
end

% solve SDP problem for computing the atomic norm using Pos3Poly
p = [n n 1];      % degree and coefficients size (scalars)
ptype = [];
ptype.trigonometric = 2;
ptype.complex_coef = 1;
ptype.brl.hsize = 1;
nhalf = n*n + (n+1)*(n+1);
cvx_begin
  variable HR(n2+nhalf) complex;
  maximize( real(x'*HR(1:n2) ) )
  subject to
    HR == sos_pol(p, ptype);
    HR(n2+1:end) == eye(nhalf,1);
cvx_end

% draw graph of trigonometric polynomial to see the frequencies
h = mat(HR(1:n2));      % coefficients of the polynomial
freqz_2d(h,1);
hold on
%scatter3( w2/pi, w1/pi, [0 0 0], 'x')
for i = 1 : length(c)
  plot3( [w2(i) w2(i)]/pi, [w1(i) w1(i)]/pi, [0 5], 'LineWidth', 2); 
end
hold off
axis([-1 1 -1 1 -60 5])

%print -deps 'fig_atom_norm_2d.eps'
