function h = factsp_orch(g)

% Spectral factorization algorithm by Newton-Raphson.
% Code by Orchard & Willson, IEEE TCAS I, vol.50, no.3, March 2003.
% Input:
%   g   - nonnegative polynomial (causal coefficients)
% Output:
%   h   - minimum phase spectral factor

% BD 9.12.2004

itmax = 100;
g = g(:);
N = length(g);
h = eye(N,1);          % initialization
ssp = realmax;
ss = ssp / 2;
iter = 0;
d = 0;
while (ss < ssp) & (iter < itmax)
  h = h + d;
  ssp = ss;
  iter = iter + 1;
  Ar = toeplitz([h(1), zeros(1,N-1)], h);
  Al = fliplr(toeplitz([h(N), zeros(1,N-1)], flipud(h)) );
  A = Al + Ar;
  b = g - Al*h;
  d = A\b;
  ss = norm(d);
end
if iter == itmax
  fprintf('Warning: maximum number of iterations attained\n')
end