function [X, hinf_err, E] = fir2_deconv_hinf(G, D, Nx, nplus)

% Find optimal Hinf solution to 2D FIR deconvolution problem.
% Input:
%   G         - given channel model (coefficient matrix)
%   D         - desired model (coefficient matrix)
%   Nx        - order of FIR equalizer
%   nplus     - relaxation degree is Nx+nplus
% Output:
%   H         - optimal filter
%   hinf_err  - optimal error norm
%   E         - error 2D FIR system

% BD 9.05.2010

% build cells for the Hinf fitting problem
for i1 = 1 : size(G,1)
  for i2 = 1 : size(G,2)
    A{i1,i2} = [G(i1,i2) 0];
  end
end
A{1,1} = [G(1,1) 1];

for i1 = 1 : size(D,1)
  for i2 = 1 : size(D,2)
    B{i1,i2} = [D(i1,i2) 0];
  end
end

[X, hinf_err, E] = fit2_infmin_left(A, B, Nx, nplus);
