function hinfn = fir2_hinf_norm(H, nplus)

% Compute (approximate) Hinf norm of a 2D FIR filter
% Method: BRL lemma based on sum-of-squares relaxation, CVX implementation
% Input:
%   H     - matrix of filter coefficients
%   nplus - extra degree for relaxation (implicit value is zero)
% Output:
%   hinfn - Hinf norm (an upper bound, actually) of the filter

% BD 5.05.2010

if nargin < 2
  nplus = [0 0];
end

% extend size if necessary
nt = size(H);
nh = nt + nplus;
Ht = zeros(nh);
Ht(1:nt(1), 1:nt(2)) = H;
nH = prod(nh);

% CVX program
cvx_begin
  v = brl_trig_poly( nh-1 );
  minimize v(1);
  subject to
    v(2:end) == vec(Ht);
cvx_end

hinfn = sqrt(v(1));
