function [H, err, E] = fir_deconv_inf_rob(G, D, Nx)

% Find optimal solution to robust FIR Hinf deconvolution problem.
% Input:
%   G         - given channel model, a polytope with nodes
%               G(z) = G{i,1} + G{i,2} z^{-1} + ...
%               with i = 1 : I
%   D         - desired model
%   Nx        - order of FIR equalizer
% Return:
%   H         - optimal filter
%   err       - worst-case error norm
%   E         - error system for each of the vertices

% BD 9.02.2006

[p,pg] = size(G{1}{1});
[m,pd] = size(D{1});
if pg ~= pd
  error('Wrong sizes')
end
if p ~= pg || m ~= pd
  error('Only square systems allowed')
end

% augment model to account for channel noise
for i = 1 : length(G);
  A{i}{1} = [G{i}{1} eye(pg)];
  for k = 2 : length(G{i})
    A{i}{k} = [G{i}{k} zeros(pg)];
  end
end
for k = 1 : length(D)
  B{k} = [D{k} zeros(pd)];
end

[H, err, E] = fit_infmin_rob_left(A, B, Nx);
