function [h,Es,r] = fir_lowpass_minEs_mag_cvx(n, wp, ws, gp, gs)

% Design (nonlinear phase) FIR filter optimizing its
% stopband energy subject to magnitude constraints
% in passband and stopband.
% Implementation with CVX.
% Input:
%   n       - filter order (must be even)
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients
%   Es      - optimal stopband energy
%   r       - causal coefficients of R(z) = H(z) * H(1/z)

% BD 7.05.2010

% the linear criterion
c = zeros(n+1,1);
c(1) = (1 - ws);
c(2:end) = - 2*sin(pi*ws*(1:n)') ./ (pi*(1:n)');

cvx_begin
  variable r(n+1);    % squared magnitude coefficients
  minimize(c'*r);
  subject to
    (1+gp)^2 * eye(n+1,1) - r == sos_trig_poly(n);
    r - (1-gp)^2 * eye(n+1,1) == pos_trig_poly( n, 1, [0 wp*pi] );
    gs^2 * eye(n+1,1) - r     == pos_trig_poly( n, 1, [ws*pi pi] );
    r                         == sos_trig_poly(n);
cvx_end

Es = c' * r;
h = factsp_orch(r);
