function b = iir2_ls_fixa( m1, m2, a, freq1, freq2, Hid, weight)

% Find optimum numerator of degree m of an IIR filter,
% for given denominator and a WLS criterion.
% Input:
%   m1, m2	 - numerator degrees
%   a        - denominator (first coefficient is assumed to be 1)
%   freq1, freq2	- frequency mesh, L1xL2 points
%   Hid	     - desired frequency response, on the mesh
%   weight	 - the weight associated with the frequency points
% Output
%   b        - LS optimal denominator

% BD 19.02.2002

L1 = length(freq1);
L2 = length(freq2);
if [L1,L2] ~= size(Hid)
   error('Incorrect desired response vector length')
end
if [L1,L2] ~= size(weight)
   error('Incorrect weights vector length')
end

freq1 = freq1(:);
freq2 = freq2(:);

Aw = polyval_2d(a, exp(-j*freq1), exp(-j*freq2));
Aw = Aw(:);

Exp1 = exp(-j*freq1*(0:m1));
Exp2 = exp(-j*freq2*(0:m2));

weight = sqrt(weight(:));	% vectorize weights

C = kron( Exp2, Exp1 );
C = repmat( weight./Aw, 1, size(C,2) ) .* C;

d=weight.*Hid(:);		% free term

b = (real(C)'*real(C) + imag(C)'*imag(C))\real(C'*d);
b = reshape( b, m1+1, m2+1 );
