function b = iir_chebwu_fixa(m, a, freq, dH, weight, max_count)

% Iterative reweighting design of approximately minimax IIR filters with fixed denominator.
% Input:
%   m           - degree of numerator
%   a           - denominator
%   freq		    - frequency grid, L points
%   dH 	  	    - desired frequency response, on the grid
%   weight	    - the weights associated with the frequency points
%   max_count   - maximum number of iterations
% Output:
%   b           - numerator coefficients

mfact = a(1);
a = a(:)' / a(1);
a = a(2:end);

minmax_weight = weight;
weight = ones(size(weight));
if nargin < 6
  max_count = 100;
end

err = 1;
tol = 1e-3;
visual = 0;

b = iir_ls_fixa( m, [1 a], freq, dH, minmax_weight.^2);
k = 0;
while (err > tol) 
  k = k+1;
  weight = weight_update(weight, dH, freq, a, b, minmax_weight, visual);
  if visual
    title([num2str(n) ': ' num2str(k)])
    xlabel('Normalized Frequency')
    ylabel(['Frequency response error';... 
	    'and its enevelope       ']),pause(0)
  end
  
  b = iir_ls_fixa( m, [1 a], freq, dH, weight);
  
  e = dH - zpolyval(b,exp(j*freq))./zpolyval([1,a],exp(j*freq));
  ripple(k) = max(abs(e .* minmax_weight));
  
  if k > 1
    err = abs(ripple(k)-ripple(k-1))/ripple(k-1);
  end
end
maxripple=ripple(end);

b = b*mfact;
