function b = iir_ls_fixa(m, a, freq, Hid, weight)
%
% Find optimum numerator of degree m of an IIR filter,
% for given denominator and a WLS criterion.
% Input:
%   m		    - numerator degree
%   a		    - denominator
%   freq		- frequency grid, L points
%   Hid	  	- desired frequency response, on the grid
%   weight	- the weights associated with the frequency points
% Output:
%   b       - optimal numerator

% BD 07.02.2002
% RN 12.02.2002

L = length(freq);
if L ~= length(Hid)
   error('Incorrect desired response vector length')
end
if L ~= length(weight)
   error('Incorrect weights vector length')
end

a = a(:);n=length(a)-1;
Hid=Hid(:);
freq=freq(:);

Aw=zpolyval(a,exp(j*freq));
Exp=exp(-j*freq*(0:m));

M=repmat(1./Aw,1,m+1).*Exp;

b=(real(M'*diag(weight)*M)\real(M'*diag(weight)*Hid))';
