function alpha = max_alpha(delta,a,rho)

%   alpha = max_alpha(delta,a,rho)
%
%  Computes the maximal alpha for which
%
%      alpha * delta (rho z)
%  1 + ------------------
%           a (rho z)
%
% is positive real.
% fast approximate method

%  BD 31.01.2002

a=a(:);
delta=delta(:);

n = length(a);

if nargin==2
  rho=1;
end

R = cumprod(rho*ones(n,1));

a = a./R;
delta = delta./R;

fz = 0;
lz = pi;
step = (lz-fz)/500;

z = exp(-j*(fz : step : lz));
aa = fliplr([1 a']);
dd = fliplr([0 delta']);

[alpha,k] = min( real( polyval( dd, z ) ./ polyval( aa, z ) ) );
%plot( real( polyval( dd, z ) ./ polyval( aa, z ) ) );

fz = fz + (k-2)*step;
lz = fz + 2*step;
step = (lz-fz)/100;
z = exp(-j*(fz : step : lz));
[alpha,k] = min( real( polyval( dd, z ) ./ polyval( aa, z ) ) );

alpha = - 0.99 / alpha;

