function [alpha_out, crit] = min_gold_2d(a, b, freq1, freq2, dH, ...
                                 weight, delta_a, delta_b, alpha_in, fcrit, p);
% golden section for 2D IIR design

% BD 20.02.2002

eps = 1e-5;
N = max(round(log(eps/alpha_in)/log(2/3)),2);
phi=(sqrt(5)-1)/2;

if nargin < 10
  fcrit = 'Jp_2d';
  p = 2;
end

alpha(1) = 0;
alpha(4) = alpha_in;
alpha(3) = alpha(1) + phi * ( alpha(4) - alpha(1) );
alpha(2) = alpha(1) + phi * ( alpha(3) - alpha(1) );

cost(1) = feval(fcrit, a+alpha(1)*delta_a, b+alpha(1)*delta_b,...
		 freq1, freq2, dH, weight, p);
cost(2) = feval(fcrit, a+alpha(2)*delta_a, b+alpha(2)*delta_b,...
		 freq1, freq2, dH, weight, p);
cost(3) = feval(fcrit, a+alpha(3)*delta_a, b+alpha(3)*delta_b,...
		 freq1, freq2, dH, weight, p);
cost(4) = feval(fcrit, a+alpha(4)*delta_a, b+alpha(4)*delta_b,...
		 freq1, freq2, dH, weight, p);
for n=1:N,
  
  [val,i] = min(cost);
  
  if (i==1) | (i==2);
    alpha(4) = alpha(3);
    alpha(3) = alpha(2);
    alpha(2) = alpha(1) + phi * ( alpha(3) - alpha(1) );
    cost(4)  = cost(3);
    cost(3)  = cost(2);
    cost(2)  = feval(fcrit, a+alpha(2)*delta_a, b+alpha(2)*delta_b,...
		              freq1, freq2, dH, weight, 2);
  else
    alpha(1) = alpha(2);
    alpha(2) = alpha(3);
    alpha(3) = alpha(1) + phi * ( alpha(4) - alpha(1) );
    cost(1)  = cost(2);
    cost(2)  = cost(3);
    cost(3)  = feval(fcrit, a+alpha(3)*delta_a, b+alpha(3)*delta_b,...
		              freq1, freq2, dH, weight, 2);
  end
end
alpha_out = alpha(i);
crit = cost(i);
