function [mu,Q] = minpol1(R)

% [mu,Q] = minpol1(R)
%
% Compute the minimum value on the unit circle of a trigonometric polynomial
% Input:
%   R       - vector of distinct coefficients, starting with free term
% Output:
%   mu      - the minimum
%   Q       - the most positive Gram matrix associated with the polynomial

% BD 26.10.2005

n = length(R);    % length of the polynomial

Ks.f = 1;         % one free variables (mu)
Ks.s = [n];       % one positive semidefinite matrix of size nxn (Q)
nrA = n;          % number of equality constraints (one for each coefficient of r)
ncA = 1+n*n;      % number of (scalar) variables (in mu and Q)

if ~isreal(R)     % specify complex data, if this is the case
  Ks.scomplex = 1;
  Ks.ycomplex = 1:n;
end

As = sparse(nrA,ncA);
bs = R(:);        % the right hand term of the equality constraints
cs = zeros(ncA,1);
cs(1) = -1;       % the objective is to maximize mu
  
e = ones(n,1);    % generate the Toeplitz elementary matrices
for k = 1:n
  As(k,2:end) = vec( spdiags(e,k-1,n,n) )';
end
As(1,1) = 1;

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

mu = x(1);        % recover the variables
Q = mat(x(2:end)) + (mu/n)*eye(n);
