function [m,Q] = minpol1_cvx(R)

% [m,Q] = minpol1_cvx(R)
%
% Compute the minimum value on the unit circle of a trigonometric polynomial, using CVX
% Input:
%   R       - vector of distinct coefficients, starting with free term
% Output:
%   mu      - the minimum
%   Q       - the most positive Gram matrix associated with the polynomial

% BD 18.08.2016

n = length(R);    % length of the polynomial

cvx_begin
  variable m;
  if ~isreal(R)     % complex data
    variable Q(n,n) complex semidefinite;
  else              % real data
    variable Q(n,n) semidefinite;
  end
  maximize( m )
  subject to  
    m + trace(Q) == R(1);  % equality constraints
    e = ones(n,1);    % generate the Toeplitz elementary matrices
    for k = 2:n
       vec( spdiags(e,k-1,n,n) )' * vec(Q) == R(k);
    end
cvx_end
Q = Q + (m/n)*eye(n);
