function [mu,Q] = minpol1_real(P)

% Computes the minimum value of the real polynomial P.
% Input:
%   P   - vector of coefficients, in the order of increasing powers.
% Output:
%   mu  - minimum value of the polynomial
%   Q   - associated Gram matrix

% BD 4.11.2004

m = length(P);
if rem(m,2) == 0
  error('The degree must be even')
end
if P(end) <= 0
  error('Leading coefficient must be positive')
end

n = (m+1)/2;    % the size of Gram matrix

Ks.l = 1;
Ks.s = [n];
nrA = m;
ncA = 1+n*n;

As = sparse(nrA,ncA);
bs = P(:);
cs = zeros(ncA,1);
cs(1) = -1;
  
e = ones(n,1);
for i = 1:m
  H = spdiags(e,m-n-i+1,n,n);
  As(i,2:end) = vec( H(:,end:-1:1) )';     
end
As(1,1) = 1;

pars.fid = 0;
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

mu = x(1);
Q = mat(x(2:end));
Q(1,1) = Q(1,1) + mu;

