function [m,Q,S] = minpol2_g2_even_cvx(n, R)

% [m,Q,S] = minpol2_g2_even_cvx(n, R)
%
% Compute the minimum value on the unit circle of a 2-D real trigonometric polynomial
% Method: 2-Gram matrix parameterization
% Warning: works only for even degree !!!
% Input:
%   R       - polynomial coefficients, symmetric matrix (all coefficients)
% Output:
%   m       - the minimum
%   Q,S     - most positive Gram pair matrices associated with the polynomial

% BD 23.08.2016

if mod(n(1),2) == 1 || mod(n(2),1) == 1
  error('This program works only for even degree')
end
nh = floor(n/2);        % half the order
Nc = (prod(n+1)+1)/2;   % number of coefs for cos base
Ns = Nc - 1;            % number of coefs for sin base

iv = (1:Nc)';           % halfplane indices
ii = nh(1)+1;
kk = 2*n(1) + 1 - nh(1);
for i2 = 1 : nh(2)
  iv( ii+1 : ii+n(1)+1 ) = kk+1 : kk+n(1)+1;
  ii = ii + n(1)+1;
  kk = kk + 2*n(1) + 1;
end

for k = 1 : length(R)
  Phi{k} = zeros(Nc);
  Lam{k} = zeros(Ns);
end

% Phi matrices
for ii = 1 : Nc
  for ll = 1 : Nc
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    if k == 1, ad = 0.5; else, ad = 0.25; end
    Phi{k}(ii,ll) = Phi{k}(ii,ll) + ad;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    if k == 1, ad = 0.5; else, ad = 0.25; end
    Phi{k}(ii,ll) = Phi{k}(ii,ll) + ad;
  end
end

% Lambda matrices
iv = iv(2:end);      % no free term for sines
for ii = 1 : Ns
  for ll = 1 : Ns
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    if k == 1, ad = 0.5; else, ad = 0.25; end
    Lam{k}(ii,ll) = Lam{k}(ii,ll) - ad;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    if k == 1, ad = 0.5; else, ad = 0.25; end
    Lam{k}(ii,ll) = Lam{k}(ii,ll) + ad;
  end
end

cvx_begin
  variable m;
  variable Q(Nc,Nc) semidefinite;
  variable S(Ns,Ns) semidefinite;
  maximize m
  subject to
    m + vec(Phi{1})' * vec(Q) + vec(Lam{1})' * vec(S) == R(1);
    for k = 2 : length(R)
      vec(Phi{k})' * vec(Q) + vec(Lam{k})' * vec(S) == R(k);
    end
cvx_end
