function Q = most_pos_Gram1_real(P)

% Computes the most positive Gram matrix associated with the real polynomial P.
% Input:
%   P   - vector of coefficients, in the order of increasing powers.
% Output:
%   Q   - most positive Gram matrix

% BD 2.05.2010

m = length(P);
if rem(m,2) == 0
  error('The degree must be even')
end
if P(end) <= 0
  error('Leading coefficient must be positive')
end

n = (m+1)/2;    % the size of Gram matrix

Ks.l = 1;
Ks.s = [n];
nrA = m;
ncA = 1+n*n;

As = sparse(nrA,ncA);
bs = P(:);
cs = zeros(ncA,1);
cs(1) = -1;
  
e = ones(n,1);
for i = 1:m
  H = spdiags(e,m-n-i+1,n,n);
  As(i,2:end) = vec( H(:,end:-1:1) )';     
end
As(1:2:end,1) = 1;

pars.fid = 0;
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

lam = x(1);
Q = mat(x(2:end)) + lam*eye(n);
