function v = pol_cs2vec(R)

% Vectorize the coefficients of a matrix trigonometric polynomial
% given in cell (symmetric) form. Uses only the first halfplane coefficients.

% BD 13.01.2009

nd = size(R);
if prod(nd) == length(R)   % 1D polynomial
  nd = length(R);
end

nvar = length(nd);
Rsize = (nd+1)/2;
nd = Rsize - 1;     % nd is now the degree
K = size( R{1}, 1 );
K2 = K*(K+1)/2;

% total number of scalar coefficients
N = (prod( 2*nd+1 ) + 1) / 2;     % number of distinct block coefs
M = N * K^2 - K*(K-1)/2;
v = zeros(M,1);

v(1:K2) = vecs(R{N});
k = K2;
ki = zeros(nvar,1);
for kk = N-1 : -1 : 1
  ki(1) = ki(1) + 1;        % go to next (poly)index
  for i = 1 : nvar - 1
    if ki(i) == Rsize(i)
      ki(i) = -Rsize(i)+1;
      ki(i+1) = ki(i+1) + 1;
    else
      break;
    end
  end
  v( k+1 : k+K*K ) = vec(R{kk});
  k = k+K*K;
end
