function y = polyval_2d(A, x1, x2)
%
% y = polyval(A, x1, x2)
%
% Computes the value of a 2D polynomial.
% Arguments:
% A		- the 2D polynomial
% x1, x2	- where to evaluate
%
% Return: y = A(x1,x2).
%
% If x1, x2 are vectors, y is a matrix with the values of A
% in each pair of the "cartesian product" of x1 and x2.

% BD 19.02.2002

[n1, n2] = size(A);

x1 = x1(:);
x2 = x2(:);

X1 = repmat(x1.',n1,1);
X1(1,:) = ones(1,length(x1));
X1 = cumprod(X1);
X2 = repmat(x2.',n2,1);
X2(1,:) = ones(1,length(x2));
X2 = cumprod(X2);

y = X1.' * A * X2;
