function [failnr, d1] = stab2wlb( A, tol )
%
% 2D stability test.
% Arguments:
% A	- the 2D quarter plane polynomial, with negative powers of z_1, z_2
% tol	- tolerance to decide if roots on the unit circle
%
% Return:
% failnr	- 0: the polynomial is stable
%	  	  1,2,3: not stable, failnr is the number of failed test
%		  (first two tests are 1D, the third is 2D)
% d1		- distance to the unit circle of the eigenvalues
%
% Algorithm of Wang, Lee, Boley, IEEE Trans. Circ. Syst. I, March '92.

% BD 17.02.2002

if nargin < 2
  tol = 1e-7;
end

failnr = 0;

[n1,n2] = size(A);
n1 = n1-1;
n2 = n2-1;

As = A(end:-1:1, end:-1:1);

% 1D tests
%r = roots(A(1,:));         % (small) error in original paper !!!
r = roots(sum(A'));
d1 = 1 - max(abs(r));
if d1 < tol
  failnr = 1;
  return
end

%r = roots(A(:,1));
r = roots(sum(A));
d1 = 1 - max(abs(r));
if d1 < tol
  failnr = 2;
  return
end

% 2D test
N = 2 * n1 * n2;
E = eye(N);
F = zeros(N);
for k = 1 : n1-1
  F( 2*k*n2+1 : 2*(k+1)*n2, 2*(k-1)*n2+1 : 2*k*n2 ) = eye(2*n2);
end

for k = 1 : n1+1
  S1 = toeplitz([A(k,1); zeros(n2-1,1)], [A(k,:) zeros(1,n2-1)]);
  S2 = toeplitz([As(k,1); zeros(n2-1,1)], [As(k,:) zeros(1,n2-1)]);
  S{k} = [S1 ; S2];
end

lcol = N - 2*n2+1 : N;
E(lcol, lcol) = S{1};

for k = 1 : n1-1
  F( 2*k*n2+1 : 2*(k+1)*n2, lcol ) = -S{n1+1-k};
end

%F(1:2*n2, lcol) = - S{n1+1} * S{n1+1};
F(1:2*n2, lcol) = - S{n1+1};

r = eig(E,F);
d1 = min(abs( 1 - abs(r) ));
if d1 < tol
  failnr = 3;
end

