function [is_stable, mineig, gap] = stab_fm(A1, A2, A3, nplus)

% Stability test for a Fornasini Marchesini model,
% using positive polynomials.
% Input:
%   A1, A2, A3    - model matrices
%   nplus         - extra degree for relaxation
% Output:
%   is_stable     - 1 if system is stable, 0 if not
%   mineig        - minimum eigenvalue of Gram matrix
%   gap           - gap (between primal and dual) achieved by the SDP solver

% BD 13.02.2006

if nargin < 4
  nplus = 0;
end
nb = size(A1,1);    % size of the blocks

% 1D test
if abs(eig(abs(A1))) >= 1 | abs(eig(abs(A2))) >= 1
  is_stable = 0;
  mineig = 0;
  gap = 1;
  return
end
e1 = max(abs(eig((eye(nb)-A2)\(A1+A3))));
e2 = max(abs(eig((eye(nb)-A1)\(A2+A3))));
if max(e1,e2) > 1   % unstable !
  is_stable = 0;
  mineig = 0;
  gap = 1;
  return
end
  
% form the positive polynomial
Rsize = [2 2] + nplus;
for i = 1 : 2*Rsize(1) - 1    % initialize with zeros
  for j = 1 : 2*Rsize(2) - 1
    polcoef{i,j} = zeros( nb );
  end
end

polcoef{ 0 + Rsize(1), 0 + Rsize(2)} = eye(nb) + A1*A1' + A2*A2' + A3*A3';
polcoef{ -1 + Rsize(1), 0 + Rsize(2)} = - A1 + A3*A2';
polcoef{ 0 + Rsize(1), -1 + Rsize(2)} = - A2 + A3*A1';
polcoef{ -1 + Rsize(1), -1 + Rsize(2)} = - A3;
polcoef{ 1 + Rsize(1), -1 + Rsize(2)} = A2*A1';

[Q, mineig, minpol, gap] = most_pos_Gram_M(polcoef, Rsize-1);
if mineig > 100*gap
  is_stable = 1;
else
  is_stable = 0;
end
