function [rho] = stab_marg_wlb2(polcoef, rhomax, tol)

% Stability margin computation for discrete 2D-polynomial.
% Use bisection (between 1 and rhomax, with final accuracy tol)
% and the stability test of Wang, Lee & Boley.

% BD 15.10.2003

t0 = cputime;
[n1,n2] = size(polcoef);

if nargin < 2
  rhomax = 2;
end
if nargin < 3
  tol = 1e-3;
end

% start bisection (assume that system is stable...)
rhomin = 1;
while (rhomax-rhomin)/rhomin > tol
  rho = (rhomin + rhomax)/2;      % bisection
  
  % compute new polynomial A(rho z)
  rhov1 = rho*ones(n1,1);
  rhov1(1) = 1;
  rhov1 = cumprod(rhov1);
  rhov2 = rho*ones(n2,1);
  rhov2(1) = 1;
  rhov2 = rhov1(end) * cumprod(rhov2);
  R = hankel(rhov1,rhov2);
  polcoefrho = polcoef .* R;

  [f,d] = stab2wlb(polcoefrho);     % f=0 means stable

  % prepare bisection
  if f
    rhomax = rho;
  else
    rhomin = rho;
  end
end

total_time = cputime - t0;
