function weight=weight_update(weight,dH,freq,a,b,minmax_weight,visual)
% weight=weight_update(weight,dH,freq,a,b,minmax_weight)
%
% Lim's algorithm for quasi-equiripple filter design
%
% weight - LS weights to be updated
% dH     - prototype filter
% freq   - frequencies
% a,b    - parameters of an iir filter
% minmax_weight - the desired weights for bands

% Riitta Niemist

tol=0.01;

e = dH-zpolyval(b,exp(j*freq))./zpolyval([1,a],exp(j*freq));
v = abs(e);
vj = zeros(size(v));

if nargin==6,
  visual=0;
end

I0=1;
for k=2:length(dH)
  if  abs(abs(dH(k))-abs(dH(k-1))) < tol
    I1=k;
  else
    vj(I0)=1;
    vj(I1)=1;
    for i=I0+1:I1-1
      if (v(i) > v(i-1)) & (v(i) > v(i+1))
      	vj(i)=1;
      end
    end
    I0=I1+1;
  end
end
vj(I0)=1;
vj(I1)=1;
for i=I0+1:I1-1
  if (v(i) > v(i-1)) & (v(i) > v(i+1))
    vj(i)=1;
  end
end

B=interp1(freq(find(vj)),v(find(vj)),freq,'linear');

if visual
  plot(freq/pi,v,freq(find(vj))/pi,v(find(vj)),'r*',freq/pi,B,'r')
end

A=1/length(dH)*sum((minmax_weight./B));

weight = weight.*(A*B.*minmax_weight).^1.5;

